@@ -29,7 +29,6 @@
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
-#include <gconf/gconf-client.h>
 
 #include <libgnomekbd/gkbd-status.h>
 #include <libgnomekbd/gkbd-keyboard-drawing.h>
@@ -37,12 +36,14 @@
 #include <libgnomekbd/gkbd-keyboard-config.h>
 #include <libgnomekbd/gkbd-util.h>
 
-#include "gsd-xmodmap.h"
 #include "gsd-keyboard-xkb.h"
 #include "delayed-dialog.h"
 #include "gnome-settings-profile.h"
 
-#define GTK_RESPONSE_PRINT 2
+#define SETTINGS_KEYBOARD_DIR "org.gnome.settings-daemon.plugins.keyboard"
+
+#define DISABLE_INDICATOR_KEY "disable-indicator"
+#define SHOW_KEYBOARD_LEDS_INDICATOR_KEY "show-keyboard-leds-indicator"
 
 static GsdKeyboardManager *manager = NULL;
 
@@ -57,50 +58,43 @@ static GkbdKeyboardConfig initial_sys_kbd_config;
 
 static gboolean inited_ok = FALSE;
 
-static guint notify_desktop = 0;
-static guint notify_keyboard = 0;
+static GSettings *settings_plugin = NULL;
+static GSettings *settings_desktop = NULL;
+static GSettings *settings_keyboard = NULL;
 
 static PostActivationCallback pa_callback = NULL;
 static void *pa_callback_user_data = NULL;
 
-static const char KNOWN_FILES_KEY[] =
-    "/desktop/gnome/peripherals/keyboard/general/known_file_list";
-
 static const char *gdm_keyboard_layout = NULL;
 
 static GtkStatusIcon *icon = NULL;
 
 static GHashTable *preview_dialogs = NULL;
 
-#define noGSDKX
+static Atom caps_lock;
+static Atom num_lock;
+static Atom scroll_lock;
 
-#ifdef GSDKX
-static FILE *logfile;
+static GtkStatusIcon *indicator_icons[3];
+static const gchar *indicator_on_icon_names[] = {
+	"kbd-scrolllock-on",
+	"kbd-numlock-on",
+	"kbd-capslock-on"
+};
 
-static void
-gsd_keyboard_log_appender (const char file[],
-			   const char function[],
-			   int level, const char format[], va_list args)
-{
-	time_t now = time (NULL);
-	fprintf (logfile, "[%08ld,%03d,%s:%s/] \t", now,
-		 level, file, function);
-	vfprintf (logfile, format, args);
-	fflush (logfile);
-}
-#endif
+static const gchar *indicator_off_icon_names[] = {
+	"kbd-scrolllock-off",
+	"kbd-numlock-off",
+	"kbd-capslock-off"
+};
 
 static void
 activation_error (void)
 {
-	char const *vendor = ServerVendor (GDK_DISPLAY ());
-	int release = VendorRelease (GDK_DISPLAY ());
+	char const *vendor;
 	GtkWidget *dialog;
-	gboolean badXFree430Release;
 
-	badXFree430Release = (vendor != NULL)
-	    && (0 == strcmp (vendor, "The XFree86 Project, Inc"))
-	    && (release / 100000 == 403);
+	vendor = ServerVendor (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()));
 
 	/* VNC viewers will not work, do not barrage them with warnings */
 	if (NULL != vendor && NULL != strstr (vendor, "VNC"))
@@ -112,25 +106,16 @@ activation_error (void)
 						     GTK_BUTTONS_CLOSE,
 						     _
 						     ("Error activating XKB configuration.\n"
-						      "It can happen under various circumstances:\n"
-						      "- a bug in libxklavier library\n"
-						      "- a bug in X server (xkbcomp, xmodmap utilities)\n"
-						      "- X server with incompatible libxkbfile implementation\n\n"
-						      "X server version data:\n%s\n%d\n%s\n"
-						      "If you report this situation as a bug, please include:\n"
-						      "- The result of <b>%s</b>\n"
-						      "- The result of <b>%s</b>"),
-						     vendor,
-						     release,
-						     badXFree430Release
-						     ?
-						     _
-						     ("You are using XFree 4.3.0.\n"
-						      "There are known problems with complex XKB configurations.\n"
-						      "Try using a simpler configuration or taking a fresher version of XFree software.")
-						     : "",
+						      "There can be various reasons for that.\n\n"
+						      "If you report this situation as a bug, include the results of\n"
+						      " • <b>%s</b>\n"
+						      " • <b>%s</b>\n"
+						      " • <b>%s</b>\n"
+						      " • <b>%s</b>"),
 						     "xprop -root | grep XKB",
-						     "gconftool-2 -R /desktop/gnome/peripherals/keyboard/kbd");
+						     "gsettings get org.gnome.libgnomekbd.keyboard model",
+						     "gsettings get org.gnome.libgnomekbd.keyboard layouts",
+						     "gsettings get org.gnome.libgnomekbd.keyboard options");
 	g_signal_connect (dialog, "response",
 			  G_CALLBACK (gtk_widget_destroy), NULL);
 	gsd_delayed_show_dialog (dialog);
@@ -139,14 +124,22 @@ activation_error (void)
 static void
 apply_desktop_settings (void)
 {
+	gboolean show_leds;
+	guint i;
 	if (!inited_ok)
 		return;
 
 	gsd_keyboard_manager_apply_settings (manager);
-	gkbd_desktop_config_load_from_gconf (&current_config);
+	gkbd_desktop_config_load (&current_config);
 	/* again, probably it would be nice to compare things
 	   before activating them */
 	gkbd_desktop_config_activate (&current_config);
+
+	show_leds = g_settings_get_boolean (settings_plugin, SHOW_KEYBOARD_LEDS_INDICATOR_KEY);
+	for (i = 0; i < G_N_ELEMENTS (indicator_icons); i++) {
+		gtk_status_icon_set_visible (indicator_icons[i],
+					     show_leds);
+	}
 }
 
 static void
@@ -166,192 +159,45 @@ popup_menu_launch_capplet ()
 	}
 }
 
-static void
-show_layout_response (GtkWidget * dialog, gint resp)
-{
-	GdkRectangle rect;
-	GtkWidget *kbdraw;
-	const gchar *groupName;
-
-	switch (resp) {
-	case GTK_RESPONSE_HELP:
-		gtk_show_uri (gtk_widget_get_screen (GTK_WIDGET (dialog)),
-			      "ghelp:gswitchit?layout-view",
-			      gtk_get_current_event_time (), NULL);
-		return;
-	case GTK_RESPONSE_CLOSE:
-		gtk_window_get_position (GTK_WINDOW (dialog), &rect.x,
-					 &rect.y);
-		gtk_window_get_size (GTK_WINDOW (dialog), &rect.width,
-				     &rect.height);
-		gkbd_preview_save_position (&rect);
-		gtk_widget_destroy (dialog);
-		break;
-	case GTK_RESPONSE_PRINT:
-		kbdraw =
-		    GTK_WIDGET (g_object_get_data
-				(G_OBJECT (dialog), "kbdraw"));
-		groupName =
-		    (const gchar *) g_object_get_data (G_OBJECT (dialog),
-						       "groupName");
-		gkbd_keyboard_drawing_print (GKBD_KEYBOARD_DRAWING
-					     (kbdraw), GTK_WINDOW (dialog),
-					     groupName ? groupName :
-					     _("Unknown"));
-	}
-}
-
 static void
 show_layout_destroy (GtkWidget * dialog, gint group)
 {
-	GtkBuilder *builder =
-	    GTK_BUILDER (g_object_get_data
-			 (G_OBJECT (dialog), "builderData"));
-	g_object_unref (G_OBJECT (builder));
 	g_hash_table_remove (preview_dialogs, GINT_TO_POINTER (group));
 }
 
 static void
 popup_menu_show_layout ()
 {
-	static GkbdKeyboardDrawingGroupLevel groupsLevels[] = { {
-								 0, 1}, {
-									 0,
-									 3},
-	{
-	 0, 0}, {
-		 0, 2}
-	};
-	static GkbdKeyboardDrawingGroupLevel *pGroupsLevels[] = {
-		groupsLevels, groupsLevels + 1, groupsLevels + 2,
-		groupsLevels + 3
-	};
-
-	GtkBuilder *builder;
-	GtkWidget *dialog, *kbdraw;
-	XkbComponentNamesRec component_names;
-	XklConfigRec *xkl_data;
-	GdkRectangle *rect;
-	GError *error = NULL;
-
-	XklEngine *engine = xkl_engine_get_instance (GDK_DISPLAY ());
+	GtkWidget *dialog;
+	XklEngine *engine =
+	    xkl_engine_get_instance (GDK_DISPLAY_XDISPLAY
+				     (gdk_display_get_default ()));
 	XklState *xkl_state = xkl_engine_get_current_state (engine);
-	gchar **group_names = gkbd_status_get_group_names ();
 	gpointer p = g_hash_table_lookup (preview_dialogs,
 					  GINT_TO_POINTER
 					  (xkl_state->group));
+	gchar **group_names = gkbd_status_get_group_names ();
+
+	if (xkl_state->group < 0
+	    || xkl_state->group >= g_strv_length (group_names)) {
+		return;
+	}
+
 	if (p != NULL) {
 		/* existing window */
 		gtk_window_present (GTK_WINDOW (p));
 		return;
 	}
 
-	builder = gtk_builder_new ();
-	gtk_builder_add_from_file (builder, DATADIR "/show-layout.ui",
-				   &error);
-
-	if (error) {
-		g_error ("building ui from %s failed: %s",
-			 DATADIR "/show-layout.ui", error->message);
-		g_clear_error (&error);
-	}
-
-
 	dialog =
-	    GTK_WIDGET (gtk_builder_get_object
-			(builder, "gswitchit_layout_view"));
-	kbdraw = gkbd_keyboard_drawing_new ();
-
-	if (xkl_state->group >= 0 &&
-	    xkl_state->group < g_strv_length (group_names)) {
-		char title[128] = "";
-		snprintf (title, sizeof (title),
-			  _("Keyboard Layout \"%s\""),
-			  group_names[xkl_state->group]);
-		gtk_window_set_title (GTK_WINDOW (dialog), title);
-		g_object_set_data_full (G_OBJECT (dialog), "group_name",
-					g_strdup (group_names
-						  [xkl_state->group]),
-					g_free);
-	}
-
-	gkbd_keyboard_drawing_set_groups_levels (GKBD_KEYBOARD_DRAWING
-						 (kbdraw), pGroupsLevels);
-
-	xkl_data = xkl_config_rec_new ();
-	if (xkl_config_rec_get_from_server (xkl_data, engine)) {
-		int num_layouts = g_strv_length (xkl_data->layouts);
-		int num_variants = g_strv_length (xkl_data->variants);
-		if (xkl_state->group >= 0 &&
-		    xkl_state->group < num_layouts &&
-		    xkl_state->group < num_variants) {
-			char *l =
-			    g_strdup (xkl_data->layouts[xkl_state->group]);
-			char *v =
-			    g_strdup (xkl_data->variants
-				      [xkl_state->group]);
-			char **p;
-			int i;
-
-			if ((p = xkl_data->layouts) != NULL)
-				for (i = num_layouts; --i >= 0;)
-					g_free (*p++);
-
-			if ((p = xkl_data->variants) != NULL)
-				for (i = num_variants; --i >= 0;)
-					g_free (*p++);
-
-			xkl_data->layouts =
-			    g_realloc (xkl_data->layouts,
-				       sizeof (char *) * 2);
-			xkl_data->variants =
-			    g_realloc (xkl_data->variants,
-				       sizeof (char *) * 2);
-			xkl_data->layouts[0] = l;
-			xkl_data->variants[0] = v;
-			xkl_data->layouts[1] = xkl_data->variants[1] =
-			    NULL;
-		}
-
-		if (xkl_xkb_config_native_prepare
-		    (engine, xkl_data, &component_names)) {
-			gkbd_keyboard_drawing_set_keyboard
-			    (GKBD_KEYBOARD_DRAWING (kbdraw),
-			     &component_names);
-			xkl_xkb_config_native_cleanup (engine,
-						       &component_names);
-		}
-	}
-	g_object_unref (G_OBJECT (xkl_data));
-
-	g_object_set_data (G_OBJECT (dialog), "builderData", builder);
-	g_signal_connect (GTK_OBJECT (dialog),
-			  "destroy", G_CALLBACK (show_layout_destroy),
+	    gkbd_keyboard_drawing_new_dialog (xkl_state->group,
+					      group_names
+					      [xkl_state->group]);
+	g_signal_connect (dialog, "destroy",
+			  G_CALLBACK (show_layout_destroy),
 			  GINT_TO_POINTER (xkl_state->group));
-	g_signal_connect (G_OBJECT (dialog), "response",
-			  G_CALLBACK (show_layout_response), NULL);
-
-	rect = gkbd_preview_load_position ();
-	if (rect != NULL) {
-		gtk_window_move (GTK_WINDOW (dialog), rect->x, rect->y);
-		gtk_window_resize (GTK_WINDOW (dialog), rect->width,
-				   rect->height);
-		g_free (rect);
-	} else
-		gtk_window_resize (GTK_WINDOW (dialog), 700, 400);
-
-	gtk_window_set_resizable (GTK_WINDOW (dialog), TRUE);
-
-	gtk_container_add (GTK_CONTAINER
-			   (gtk_builder_get_object
-			    (builder, "preview_vbox")), kbdraw);
-
-	g_object_set_data (G_OBJECT (dialog), "kbdraw", kbdraw);
-
 	g_hash_table_insert (preview_dialogs,
 			     GINT_TO_POINTER (xkl_state->group), dialog);
-
-	gtk_widget_show_all (GTK_WIDGET (dialog));
 }
 
 static void
@@ -389,7 +235,7 @@ status_icon_popup_menu_cb (GtkStatusIcon * icon, guint button, guint time)
 	int i = 0;
 	gchar **current_name = gkbd_status_get_group_names ();
 
-	GtkWidget *item = gtk_menu_item_new_with_mnemonic (_("_Groups"));
+	GtkWidget *item = gtk_menu_item_new_with_mnemonic (_("_Layouts"));
 	gtk_widget_show (item);
 	gtk_menu_shell_append (GTK_MENU_SHELL (popup_menu), item);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item),
@@ -402,7 +248,7 @@ status_icon_popup_menu_cb (GtkStatusIcon * icon, guint button, guint time)
 			  NULL);
 	gtk_menu_shell_append (GTK_MENU_SHELL (popup_menu), item);
 
-	item = gtk_menu_item_new_with_mnemonic (_("Show Current _Layout"));
+	item = gtk_menu_item_new_with_mnemonic (_("Show _Current Layout"));
 	gtk_widget_show (item);
 	g_signal_connect (item, "activate", popup_menu_show_layout, NULL);
 	gtk_menu_shell_append (GTK_MENU_SHELL (popup_menu), item);
@@ -445,10 +291,17 @@ status_icon_popup_menu_cb (GtkStatusIcon * icon, guint button, guint time)
 static void
 show_hide_icon ()
 {
-	if (g_slist_length (current_kbd_config.layouts_variants) > 1) {
+	if (g_strv_length (current_kbd_config.layouts_variants) > 1) {
 		if (icon == NULL) {
+			gboolean disable =
+			    g_settings_get_boolean (settings_plugin,
+						    DISABLE_INDICATOR_KEY);
+			if (disable)
+				return;
+
 			xkl_debug (150, "Creating new icon\n");
 			icon = gkbd_status_new ();
+			gtk_status_icon_set_name (icon, "keyboard");
 			g_signal_connect (icon, "popup-menu",
 					  G_CALLBACK
 					  (status_icon_popup_menu_cb),
@@ -472,7 +325,6 @@ try_activating_xkb_config_if_new (GkbdKeyboardConfig *
 	if (!gkbd_keyboard_config_equals
 	    (&current_kbd_config, current_sys_kbd_config)) {
 		if (gkbd_keyboard_config_activate (&current_kbd_config)) {
-			show_hide_icon ();
 			if (pa_callback != NULL) {
 				(*pa_callback) (pa_callback_user_data);
 				return TRUE;
@@ -490,8 +342,7 @@ filter_xkb_config (void)
 	XklConfigItem *item;
 	gchar *lname;
 	gchar *vname;
-	GSList *lv;
-	GSList *filtered;
+	gchar **lv;
 	gboolean any_change = FALSE;
 
 	xkl_debug (100, "Filtering configuration against the registry\n");
@@ -507,27 +358,18 @@ filter_xkb_config (void)
 	}
 	lv = current_kbd_config.layouts_variants;
 	item = xkl_config_item_new ();
-	while (lv) {
-		xkl_debug (100, "Checking [%s]\n", lv->data);
-		if (gkbd_keyboard_config_split_items
-		    (lv->data, &lname, &vname)) {
+	while (*lv) {
+		xkl_debug (100, "Checking [%s]\n", *lv);
+		if (gkbd_keyboard_config_split_items (*lv, &lname, &vname)) {
+			gboolean should_be_dropped = FALSE;
 			g_snprintf (item->name, sizeof (item->name), "%s",
 				    lname);
 			if (!xkl_config_registry_find_layout
 			    (xkl_registry, item)) {
 				xkl_debug (100, "Bad layout [%s]\n",
 					   lname);
-				filtered = lv;
-				lv = lv->next;
-				g_free (filtered->data);
-				current_kbd_config.layouts_variants =
-				    g_slist_delete_link
-				    (current_kbd_config.layouts_variants,
-				     filtered);
-				any_change = TRUE;
-				continue;
-			}
-			if (vname) {
+				should_be_dropped = TRUE;
+			} else if (vname) {
 				g_snprintf (item->name,
 					    sizeof (item->name), "%s",
 					    vname);
@@ -536,20 +378,16 @@ filter_xkb_config (void)
 					xkl_debug (100,
 						   "Bad variant [%s(%s)]\n",
 						   lname, vname);
-					filtered = lv;
-					lv = lv->next;
-					g_free (filtered->data);
-					current_kbd_config.layouts_variants
-					    =
-					    g_slist_delete_link
-					    (current_kbd_config.layouts_variants,
-					     filtered);
-					any_change = TRUE;
-					continue;
+					should_be_dropped = TRUE;
 				}
 			}
+			if (should_be_dropped) {
+				gkbd_strv_behead (lv);
+				any_change = TRUE;
+				continue;
+			}
 		}
-		lv = lv->next;
+		lv++;
 	}
 	g_object_unref (item);
 	return any_change;
@@ -558,7 +396,6 @@ filter_xkb_config (void)
 static void
 apply_xkb_settings (void)
 {
-	GConfClient *conf_client;
 	GkbdKeyboardConfig current_sys_kbd_config;
 	int group_to_activate = -1;
 	char *gdm_layout;
@@ -567,18 +404,16 @@ apply_xkb_settings (void)
 	if (!inited_ok)
 		return;
 
-	conf_client = gconf_client_get_default ();
-
 	/* With GDM the user can already set a layout from the login
 	 * screen. Try to keep that setting.
 	 * We clear gdm_keyboard_layout early, so we don't risk
-	 * recursion from gconf notification.
+	 * recursion from conf notification.
 	 */
 	gdm_layout = g_strdup (gdm_keyboard_layout);
 	gdm_keyboard_layout = NULL;
 
 	/* gdm's configuration and $GDM_KEYBOARD_LAYOUT separates layout and
-	 * variant with a space, but gconf uses tabs; so convert to be robust
+	 * variant with a space, but gsettings use tabs; so convert to be robust
 	 * with both */
 	for (s = gdm_layout; s && *s; ++s) {
 		if (*s == ' ') {
@@ -587,14 +422,37 @@ apply_xkb_settings (void)
 	}
 
 	if (gdm_layout != NULL) {
-		GSList *layouts;
-		GSList *found_node;
+		gchar **layouts;
+		gboolean found_node;
 		int max_groups;
+		gchar **i;
 
-		max_groups = MAX (xkl_engine_get_max_num_groups (xkl_engine), 1);
-		layouts = gconf_client_get_list (conf_client,
-						 GKBD_KEYBOARD_CONFIG_KEY_LAYOUTS,
-						 GCONF_VALUE_STRING, NULL);
+		max_groups =
+		    MAX (xkl_engine_get_max_num_groups (xkl_engine), 1);
+		layouts =
+		    g_settings_get_strv (settings_keyboard,
+					 GKBD_KEYBOARD_CONFIG_KEY_LAYOUTS);
+
+		/* Use system layouts as a default if we do not have
+		 * user configuration */
+		if (layouts == NULL) {
+			int len;
+
+			i = layouts =
+			    g_strdupv
+			    (initial_sys_kbd_config.layouts_variants);
+			if (i != NULL) {
+				while (*i != NULL) {
+					s = *i;
+
+					/* chop off empty variants to avoid duplicates */
+					len = strlen (s);
+					if (s[len - 1] == '\t')
+						s[len - 1] = '\0';
+					i++;
+				}
+			}
+		}
 
 		/* Add the layout if it doesn't already exist. XKB limits the
 		 * total number of layouts. If we already have the maximum
@@ -602,45 +460,45 @@ apply_xkb_settings (void)
 		 * prevents the list from becoming full if the user has a habit
 		 * of selecting many different keyboard layouts in GDM. */
 
-		found_node =
-		    g_slist_find_custom (layouts, gdm_layout,
-					 (GCompareFunc) g_strcmp0);
+		found_node = FALSE;
+		i = layouts;
+		if (i != NULL) {
+			while (*i != NULL) {
+				if (!g_strcmp0 (*i, gdm_layout)) {
+					found_node = TRUE;
+					break;
+				}
+				i++;
+			}
+		}
 
 		if (!found_node) {
 			/* Insert at the last valid place, or at the end of
 			 * list, whichever comes first */
-			layouts =
-			    g_slist_insert (layouts, g_strdup (gdm_layout),
-					    max_groups - 1);
-			if (g_slist_length (layouts) > max_groups) {
-				GSList *last;
-				GSList *free_layouts;
-
-				last =
-				    g_slist_nth (layouts, max_groups - 1);
-				free_layouts = last->next;
-				last->next = NULL;
-
-				g_slist_foreach (free_layouts,
-						 (GFunc) g_free, NULL);
-				g_slist_free (free_layouts);
+			gint old_length = g_strv_length (layouts);
+			if (old_length >= max_groups) {
+				layouts[max_groups - 1] =
+				    g_strdup (gdm_layout);
+			} else {
+				layouts =
+				    gkbd_strv_append (layouts,
+						      g_strdup
+						      (gdm_layout));
 			}
 
-			gconf_client_set_list (conf_client,
-					       GKBD_KEYBOARD_CONFIG_KEY_LAYOUTS,
-					       GCONF_VALUE_STRING, layouts,
-					       NULL);
+			g_settings_set_strv (settings_keyboard,
+					     GKBD_KEYBOARD_CONFIG_KEY_LAYOUTS,
+					     (const gchar *
+					      const *) layouts);
 		}
 
-		g_slist_foreach (layouts, (GFunc) g_free, NULL);
-		g_slist_free (layouts);
+		g_strfreev (layouts);
 	}
 
-	gkbd_keyboard_config_init (&current_sys_kbd_config,
-				   conf_client, xkl_engine);
+	gkbd_keyboard_config_init (&current_sys_kbd_config, xkl_engine);
 
-	gkbd_keyboard_config_load_from_gconf (&current_kbd_config,
-					      &initial_sys_kbd_config);
+	gkbd_keyboard_config_load (&current_kbd_config,
+				   &initial_sys_kbd_config);
 
 	gkbd_keyboard_config_load_from_x_current (&current_sys_kbd_config,
 						  NULL);
@@ -666,16 +524,22 @@ apply_xkb_settings (void)
 		/* If there are multiple layouts,
 		 * try to find the one closest to the gdm layout
 		 */
-		GSList *l;
+		gchar **l;
 		int i;
 		size_t len = strlen (gdm_layout);
-		for (i = 0, l = current_kbd_config.layouts_variants; l;
-		     i++, l = l->next) {
-			char *lv = l->data;
-			if (strncmp (lv, gdm_layout, len) == 0
-			    && (lv[len] == '\0' || lv[len] == '\t')) {
-				group_to_activate = i;
-				break;
+		l = current_kbd_config.layouts_variants;
+		if (l != NULL) {
+			i = 0;
+			while (*l != NULL) {
+				char *lv = *l;
+				if (strncmp (lv, gdm_layout, len) == 0
+				    && (lv[len] == '\0'
+					|| lv[len] == '\t')) {
+					group_to_activate = i;
+					break;
+				}
+				i++;
+				l++;
 			}
 		}
 	}
@@ -686,99 +550,18 @@ apply_xkb_settings (void)
 		xkl_engine_lock_group (current_config.engine,
 				       group_to_activate);
 	gkbd_keyboard_config_term (&current_sys_kbd_config);
+	show_hide_icon ();
 }
 
 static void
 gsd_keyboard_xkb_analyze_sysconfig (void)
 {
-	GConfClient *conf_client;
-
 	if (!inited_ok)
 		return;
 
-	conf_client = gconf_client_get_default ();
-	gkbd_keyboard_config_init (&initial_sys_kbd_config,
-				   conf_client, xkl_engine);
+	gkbd_keyboard_config_init (&initial_sys_kbd_config, xkl_engine);
 	gkbd_keyboard_config_load_from_x_initial (&initial_sys_kbd_config,
 						  NULL);
-	g_object_unref (conf_client);
-}
-
-static gboolean
-gsd_chk_file_list (void)
-{
-	GDir *home_dir;
-	const char *fname;
-	GSList *file_list = NULL;
-	GSList *last_login_file_list = NULL;
-	GSList *tmp = NULL;
-	GSList *tmp_l = NULL;
-	gboolean new_file_exist = FALSE;
-	GConfClient *conf_client;
-
-	home_dir = g_dir_open (g_get_home_dir (), 0, NULL);
-	while ((fname = g_dir_read_name (home_dir)) != NULL) {
-		if (g_strrstr (fname, "modmap")) {
-			file_list =
-			    g_slist_append (file_list, g_strdup (fname));
-		}
-	}
-	g_dir_close (home_dir);
-
-	conf_client = gconf_client_get_default ();
-
-	last_login_file_list = gconf_client_get_list (conf_client,
-						      KNOWN_FILES_KEY,
-						      GCONF_VALUE_STRING,
-						      NULL);
-
-	/* Compare between the two file list, currently available modmap files
-	   and the files available in the last log in */
-	tmp = file_list;
-	while (tmp != NULL) {
-		tmp_l = last_login_file_list;
-		new_file_exist = TRUE;
-		while (tmp_l != NULL) {
-			if (strcmp (tmp->data, tmp_l->data) == 0) {
-				new_file_exist = FALSE;
-				break;
-			} else {
-				tmp_l = tmp_l->next;
-			}
-		}
-		if (new_file_exist) {
-			break;
-		} else {
-			tmp = tmp->next;
-		}
-	}
-
-	if (new_file_exist) {
-		gconf_client_set_list (conf_client,
-				       KNOWN_FILES_KEY,
-				       GCONF_VALUE_STRING,
-				       file_list, NULL);
-	}
-
-	g_object_unref (conf_client);
-
-	g_slist_foreach (file_list, (GFunc) g_free, NULL);
-	g_slist_free (file_list);
-
-	g_slist_foreach (last_login_file_list, (GFunc) g_free, NULL);
-	g_slist_free (last_login_file_list);
-
-	return new_file_exist;
-
-}
-
-static void
-gsd_keyboard_xkb_chk_lcl_xmm (void)
-{
-	if (gsd_chk_file_list ()) {
-		gsd_modmap_dialog_call ();
-	}
-	gsd_load_modmap_files ();
 }
 
 void
@@ -797,16 +580,6 @@ gsd_keyboard_xkb_evt_filter (GdkXEvent * xev, GdkEvent * event)
 	return GDK_FILTER_CONTINUE;
 }
 
-static guint
-register_config_callback (GConfClient * client,
-			  const char *path, GConfClientNotifyFunc func)
-{
-	gconf_client_add_dir (client, path, GCONF_CLIENT_PRELOAD_ONELEVEL,
-			      NULL);
-	return gconf_client_notify_add (client, path, func, NULL, NULL,
-					NULL);
-}
-
 /* When new Keyboard is plugged in - reload the settings */
 static void
 gsd_keyboard_new_device (XklEngine * engine)
@@ -815,48 +588,97 @@ gsd_keyboard_new_device (XklEngine * engine)
 	apply_xkb_settings ();
 }
 
-void
-gsd_keyboard_xkb_init (GConfClient * client,
-		       GsdKeyboardManager * kbd_manager)
+static void
+gsd_keyboard_update_indicator_icons ()
 {
+	Bool state;
+	int new_state;
+	guint i;
+	Display *display =
+	    GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
+	XkbGetNamedIndicator (display, caps_lock, NULL, &state, NULL,
+			      NULL);
+	new_state = state ? 1 : 0;
+	XkbGetNamedIndicator (display, num_lock, NULL, &state, NULL, NULL);
+	new_state <<= 1;
+	new_state |= (state ? 1 : 0);
+	XkbGetNamedIndicator (display, scroll_lock, NULL, &state,
+			      NULL, NULL);
+	new_state <<= 1;
+	new_state |= (state ? 1 : 0);
+	xkl_debug (160, "Indicators state: %d\n", new_state);
+
+	for (i = 0; i < G_N_ELEMENTS (indicator_icons); i++) {
+		gtk_status_icon_set_from_icon_name (indicator_icons[i],
+						    (new_state & (1 << i))
+						    ?
+						    indicator_on_icon_names
+						    [i] :
+						    indicator_off_icon_names
+						    [i]);
+	}
+}
+
+static void
+gsd_keyboard_state_changed (XklEngine * engine, XklEngineStateChange type,
+			    gint new_group, gboolean restore)
+{
+	xkl_debug (160,
+		   "State changed: type %d, new group: %d, restore: %d.\n",
+		   type, new_group, restore);
+	if (type == INDICATORS_CHANGED) {
+		gsd_keyboard_update_indicator_icons ();
+	}
+}
+
+void
+gsd_keyboard_xkb_init (GsdKeyboardManager * kbd_manager)
+{
+	guint i;
+	Display *display =
+	    GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
 	gnome_settings_profile_start (NULL);
-#ifdef GSDKX
-	xkl_set_debug_level (200);
-	logfile = fopen ("/tmp/gsdkx.log", "a");
-	xkl_set_log_appender (gsd_keyboard_log_appender);
-#endif
+
+	settings_plugin = g_settings_new (SETTINGS_KEYBOARD_DIR);
+
+	gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (),
+					   DATADIR G_DIR_SEPARATOR_S
+					   "icons");
+
+	caps_lock = XInternAtom (display, "Caps Lock", False);
+	num_lock = XInternAtom (display, "Num Lock", False);
+	scroll_lock = XInternAtom (display, "Scroll Lock", False);
+
+	for (i = 0; i < G_N_ELEMENTS (indicator_icons); i++) {
+		indicator_icons[i] =
+		    gtk_status_icon_new_from_icon_name
+		    (indicator_off_icon_names[i]);
+	}
+
+	gsd_keyboard_update_indicator_icons ();
+
 	manager = kbd_manager;
 	gnome_settings_profile_start ("xkl_engine_get_instance");
-	xkl_engine = xkl_engine_get_instance (GDK_DISPLAY ());
+	xkl_engine = xkl_engine_get_instance (display);
 	gnome_settings_profile_end ("xkl_engine_get_instance");
 	if (xkl_engine) {
 		inited_ok = TRUE;
 
 		gdm_keyboard_layout = g_getenv ("GDM_KEYBOARD_LAYOUT");
 
-		gkbd_desktop_config_init (&current_config,
-					  client, xkl_engine);
+		gkbd_desktop_config_init (&current_config, xkl_engine);
 		gkbd_keyboard_config_init (&current_kbd_config,
-					   client, xkl_engine);
+					   xkl_engine);
 		xkl_engine_backup_names_prop (xkl_engine);
 		gsd_keyboard_xkb_analyze_sysconfig ();
-		gnome_settings_profile_start
-		    ("gsd_keyboard_xkb_chk_lcl_xmm");
-		gsd_keyboard_xkb_chk_lcl_xmm ();
-		gnome_settings_profile_end
-		    ("gsd_keyboard_xkb_chk_lcl_xmm");
 
-		notify_desktop =
-		    register_config_callback (client,
-					      GKBD_DESKTOP_CONFIG_DIR,
-					      (GConfClientNotifyFunc)
-					      apply_desktop_settings);
-
-		notify_keyboard =
-		    register_config_callback (client,
-					      GKBD_KEYBOARD_CONFIG_DIR,
-					      (GConfClientNotifyFunc)
-					      apply_xkb_settings);
+		settings_desktop = g_settings_new (GKBD_DESKTOP_SCHEMA);
+		settings_keyboard = g_settings_new (GKBD_KEYBOARD_SCHEMA);
+		g_signal_connect (settings_desktop, "changed",
+				  (GCallback) apply_desktop_settings,
+				  NULL);
+		g_signal_connect (settings_keyboard, "changed",
+				  (GCallback) apply_xkb_settings, NULL);
 
 		gdk_window_add_filter (NULL, (GdkFilterFunc)
 				       gsd_keyboard_xkb_evt_filter, NULL);
@@ -866,6 +688,9 @@ gsd_keyboard_xkb_init (GConfClient * client,
 			g_signal_connect (xkl_engine, "X-new-device",
 					  G_CALLBACK
 					  (gsd_keyboard_new_device), NULL);
+		g_signal_connect (xkl_engine, "X-state-changed",
+				  G_CALLBACK
+				  (gsd_keyboard_state_changed), NULL);
 
 		gnome_settings_profile_start ("xkl_engine_start_listen");
 		xkl_engine_start_listen (xkl_engine,
@@ -881,19 +706,28 @@ gsd_keyboard_xkb_init (GConfClient * client,
 		gnome_settings_profile_end ("apply_xkb_settings");
 	}
 	preview_dialogs = g_hash_table_new (g_direct_hash, g_direct_equal);
+
 	gnome_settings_profile_end (NULL);
 }
 
 void
 gsd_keyboard_xkb_shutdown (void)
 {
-	GConfClient *client;
+	guint i;
 
 	pa_callback = NULL;
 	pa_callback_user_data = NULL;
 	manager = NULL;
 
-	g_hash_table_destroy (preview_dialogs);
+	for (i = 0; i < G_N_ELEMENTS (indicator_icons); i++) {
+		if (indicator_icons[i] != NULL) {
+			g_object_unref (G_OBJECT (indicator_icons[i]));
+			indicator_icons[i] = NULL;
+		}
+	}
+
+	if (preview_dialogs != NULL)
+		g_hash_table_destroy (preview_dialogs);
 
 	if (!inited_ok)
 		return;
@@ -905,27 +739,17 @@ gsd_keyboard_xkb_shutdown (void)
 	gdk_window_remove_filter (NULL, (GdkFilterFunc)
 				  gsd_keyboard_xkb_evt_filter, NULL);
 
-	client = gconf_client_get_default ();
-
-	if (notify_desktop != 0) {
-		gconf_client_remove_dir (client, GKBD_DESKTOP_CONFIG_DIR,
-					 NULL);
-		gconf_client_notify_remove (client, notify_desktop);
-		notify_desktop = 0;
-	}
-
-	if (notify_keyboard != 0) {
-		gconf_client_remove_dir (client, GKBD_KEYBOARD_CONFIG_DIR,
-					 NULL);
-		gconf_client_notify_remove (client, notify_keyboard);
-		notify_keyboard = 0;
-	}
+	g_object_unref (settings_desktop);
+	settings_desktop = NULL;
+	g_object_unref (settings_keyboard);
+	settings_keyboard = NULL;
+	g_object_unref (settings_plugin);
+	settings_plugin = NULL;
 
 	if (xkl_registry) {
 		g_object_unref (xkl_registry);
 	}
 
-	g_object_unref (client);
 	g_object_unref (xkl_engine);
 
 	xkl_engine = NULL;
